@@ -79,7 +79,7 @@ sub new_from_parts {
 	$class = 'Net::SIP::Response' if $class eq 'Net::SIP::Packet';
     } else {
 	# Request
-	$code = uc($code);                             # uppercase method
+	$code = uc($code);  # uppercase method
 	$class = 'Net::SIP::Request' if $class eq 'Net::SIP::Packet';
     }
 
@@ -136,7 +136,7 @@ sub new_from_string {
     if ( $class eq 'Net::SIP::Packet' ) {
 	$class = $data->{code} =~m{^\d}
 	    ? 'Net::SIP::Response'
-	    :'Net::SIP::Request';
+	    : 'Net::SIP::Request';
     }
     my $self = fields::new($class);
     %$self = %$data;
@@ -365,7 +365,7 @@ sub set_body {
 ###########################################################################
 sub scan_header {
     my Net::SIP::Packet $self = shift;
-    my $key = _normalize_hdrkey(shift) if @_>1;
+    my $key = @_>1 ? _normalize_hdrkey(shift) : undef;
     my $sub = shift;
 
     ($sub, my @args) = ref($sub) eq 'CODE' ? ($sub):@$sub;
@@ -398,7 +398,7 @@ sub as_string {
 	$self->set_header( 'content-length',length($body))
     }
 
-    # return immediatly if request is up to date
+    # return immediately if request is up to date
     return $self->{as_string} if $self->{as_string};
 
     my $header = $self->{header};
@@ -494,7 +494,7 @@ sub dump {
 sub as_parts {
     my $self = shift;
 
-    # if parts are up to date return immediatly#
+    # if parts are up to date return immediately#
     if ( ! $self->{code} ) {
 	my $data = _string2parts( $self->{as_string} );
 	%$self = ( %$self,%$data );
